#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include "pictures.h"

//-----------------------------------------------------------------------------
// Name: drawdat
// Desc: Write the extracted pixels into the bitmap
//-----------------------------------------------------------------------------

void LZW2BMP::drawdat()
{
	int zeros = ((realwidth * 3) % 4);
	if(zeros > 0)
	{
		zeros = 4 - zeros;
	}
	for(int b = (height - 1); b >= 0; b--)
	{
		for(int a = 0; a < realwidth; a++)
		{
			writepicture(pixels[a][b]);
		}
		writezeros(zeros);
	}
}

//-----------------------------------------------------------------------------
// Name: writebitmap
// Desc: Draws the bitmap
//-----------------------------------------------------------------------------

void LZW2BMP::writebitmap(int width1, int height1, bool mask,
						  int numxtiles1 = 1, int numytiles1 = 1){
	height = height1;
	width = width1;
	numxtiles = numxtiles1;
	numytiles = numytiles1;
	tilewidth = (width1/numxtiles);
	tileheight = (height1/numytiles);
	writeheader();
	init();
	writedat(mask);
	drawdat();
	writeend();
}

//-----------------------------------------------------------------------------
// Name: writebmplzw1
// Desc: Opens *.lzw and draws its contents
//-----------------------------------------------------------------------------

void LZW2BMP::writebmplzw1(const char* bmp, int num,
						   int width1, int height1, bool mask=false)
{
	char bmpfile[_MAX_PATH];
	sprintf(bmpfile,".//lzw//%s%d.bmp",bmp,num);
	fw = fopen(bmpfile,"wb");
	if(mask==true)
	{
		width1*= 4;
	}
	writebitmap(width1,height1,mask);
	fclose(fw);
}

//-----------------------------------------------------------------------------
// Name: writedat
// Desc: Get the data from *.ch and draw the 2 pixels
//-----------------------------------------------------------------------------

void LZW2BMP::writedat(bool mask)
{
	int s=2;
	int y;
	int z;
	int q[8];
	if(mask==true)
	{
		s*=4;
	}

	for(int d = 0; d < numytiles; d++){
		for(int c = 0; c < numxtiles; c++){
			for(int b = 0; b < tileheight; b++){
				for(int a = 0; a < ((tilewidth + 1) / s); a++){
					int x = fgetc(fp);
					if(mask==false){
						y = x/(0x10);
						z = x-(y*0x10);
						pixels[(a*2)+(c*tilewidth)][b+(d*tileheight)]=y;
						pixels[(a*2+1)+(c*tilewidth)][b+(d*tileheight)]=z;
					}
					else{
						for(int e=0;e<8;e++){
							q[e] = ((x>>(7-e))&1);
							if(q[e]==0){
								q[e]=0;
							}
							else{
								q[e]=15;
							}
						}
						pixels[(a*8)+(c*tilewidth)][b+(d*tileheight)]=q[0];
						pixels[(a*8+1)+(c*tilewidth)][b+(d*tileheight)]=q[1];
						pixels[(a*8+2)+(c*tilewidth)][b+(d*tileheight)]=q[2];
						pixels[(a*8+3)+(c*tilewidth)][b+(d*tileheight)]=q[3];
						pixels[(a*8+4)+(c*tilewidth)][b+(d*tileheight)]=q[4];
						pixels[(a*8+5)+(c*tilewidth)][b+(d*tileheight)]=q[5];
						pixels[(a*8+6)+(c*tilewidth)][b+(d*tileheight)]=q[6];
						pixels[(a*8+7)+(c*tilewidth)][b+(d*tileheight)]=q[7];
					}
				}
			}
		}
	}
}

//-----------------------------------------------------------------------------
// Name: writeend
// Desc: Write the bitmap end
//-----------------------------------------------------------------------------

void LZW2BMP::writeend(){
	writezeros(2);
}

//-----------------------------------------------------------------------------
// Name: writeheader
// Desc: Writes the bitmap header
//-----------------------------------------------------------------------------

void LZW2BMP::writeheader(){
	int tempwidth = realwidth/256;
	int tempheight = height/256;
	tempwidth &= 0xff;
	tempheight &= 0xff;
	fputc(0x42,fw);
	fputc(0x4d,fw);
	writezeros(8);
	fputc(0x36,fw);
	writezeros(3);
	fputc(0x28,fw);
	writezeros(3);
	fputc((realwidth & 0xff),fw);//The width
	fputc(tempwidth,fw);//The width
	writezeros(2);
	fputc((height & 0xff),fw);//The height
	fputc(tempheight,fw);//The height
	writezeros(2);
	fputc(0x01,fw);
	writezeros(1);
	fputc(0x18,fw);
	writezeros(9);
	fputc(0x12,fw);
	fputc(0x0b,fw);
	writezeros(2);
	fputc(0x12,fw);
	fputc(0x0b,fw);
	writezeros(10);
}

//-----------------------------------------------------------------------------
// Name: writepicture
// Desc: Writes the corresponding color in the bitmap
//-----------------------------------------------------------------------------

void LZW2BMP::writepicture(int pic){
	switch(pic){
	case 0://white
		fputc(0,fw);
		fputc(0,fw);
		fputc(0,fw);
		break;
	case 1://dark blue
		fputc(160,fw);
		fputc(0,fw);
		fputc(0,fw);
		break;
	case 2://dark green
		fputc(0,fw);
		fputc(160,fw);
		fputc(0,fw);
		break;
	case 3://blue green
		fputc(160,fw);
		fputc(160,fw);
		fputc(0,fw);
		break;
	case 4://maroon
		fputc(0,fw);
		fputc(0,fw);
		fputc(160,fw);
		break;
	case 5://dark purple
		fputc(160,fw);
		fputc(0,fw);
		fputc(160,fw);
		break;
	case 6://brown
		fputc(0,fw);
		fputc(80,fw);
		fputc(160,fw);
		break;
	case 7://light grey
		fputc(160,fw);
		fputc(160,fw);
		fputc(160,fw);
		break;
	case 8://dark grey
		fputc(80,fw);
		fputc(80,fw);
		fputc(80,fw);
		break;
	case 9://blue
		fputc(255,fw);
		fputc(0,fw);
		fputc(0,fw);
		break;
	case 10://light green
		fputc(80,fw);
		fputc(255,fw);
		fputc(80,fw);
		break;
	case 11://light blue
		fputc(255,fw);
		fputc(255,fw);
		fputc(80,fw);
		break;
	case 12://light red
		fputc(80,fw);
		fputc(80,fw);
		fputc(255,fw);
		break;
	case 13://light purple
		fputc(255,fw);
		fputc(80,fw);
		fputc(255,fw);
		break;
	case 14://yellow
		fputc(80,fw);
		fputc(255,fw);
		fputc(255,fw);
		break;
	case 15://black
		fputc(255,fw);
		fputc(255,fw);
		fputc(255,fw);
		break;
	default:
		fputc(255,fw);
		fputc(255,fw);
		fputc(255,fw);
		break;
	}
}

//-----------------------------------------------------------------------------
// Name: writetileslzw
// Desc: Opens tiles.lzw and draws its contents
//-----------------------------------------------------------------------------

void LZW2BMP::writetileslzw()
{
	if((fp = fopen(".//lzw//tiles.lzw","rb"))==NULL)
	{
		return;
	}
	fw = fopen(".//lzw//tiles.bmp","wb");
	realwidth = 512;
	writebitmap(512,256,false,32,16);
	fclose(fw);
	fclose(fp);
}

void LZW2BMP::writegraphics(const char* picname, bool usemask)
{
	if(false == usemask)
	{
		char filename[_MAX_PATH];
		unsigned short numpics;
		unsigned long* picarray = NULL;

		system("cls");
		sprintf(filename, ".//lzw//%s.lzw", picname);
		printf("Filename = %s\n", filename);
		if((fp = fopen(filename,"rb"))==NULL)
		{
			return;
		}
		fread(&numpics, sizeof(unsigned short), 1, fp); // 16 bit
		printf("Number of pictures in file = %d\n", numpics);

		picarray = (unsigned long*)malloc(sizeof(unsigned long) * numpics);

		for(unsigned short index = 0; index < numpics; index++)
		{
			unsigned long offset; // 32 bit
			fread(&offset, sizeof(unsigned long), 1, fp);
			printf("Picture #%d offset at: %d\n", index, offset);
			picarray[index] = offset;
		}

		for(unsigned short index = 0; index < numpics; index++)
		{
			unsigned short width;
			unsigned short height;
			int bufwid;
			if(picarray[index] > 0)
			{
				fseek(fp,picarray[index],SEEK_SET);
				fread(&width, sizeof(unsigned short), 1, fp); // 16 bit
				fread(&height, sizeof(unsigned short), 1, fp); // 16 bit
				bufwid = (8 - (width % 8)) % 8;
				realwidth = width;

				writebmplzw1(picname, index, width + bufwid, height, false);
			}
		}
		free(picarray);
	}
	else
	{
		{
			char filename[_MAX_PATH];
			unsigned short numpics;
			unsigned short* picarray = NULL;

			system("cls");
			sprintf(filename, ".//lzw//%s.lzw", picname);
			printf("Filename = %s\n", filename);
			if((fp = fopen(filename,"rb"))==NULL)
			{
				return;
			}
			fread(&numpics, sizeof(unsigned short), 1, fp); // 16 bit
			numpics *= 2; // need to remember the mask
			printf("Number of pictures in file = %d\n", numpics);

			picarray = (unsigned short*)malloc(sizeof(unsigned short) * numpics);

			for(unsigned short index = 0; index < numpics; index++)
			{
				unsigned short offset; // 16 bit
				fread(&offset, sizeof(unsigned short), 1, fp);
				printf("Picture #%d offset at: %d\n", index, offset);
				picarray[index] = offset;
			}

			for(unsigned short index = 0; index < numpics; index++)
			{
				unsigned short width;
				unsigned short height;
				int bufwid;
				if(picarray[index] > 0)
				{
					fseek(fp,picarray[index],SEEK_SET);
					fread(&width, sizeof(unsigned short), 1, fp); // 16 bit
					fread(&height, sizeof(unsigned short), 1, fp); // 16 bit
					bufwid = (8 - (width % 8)) % 8;
					realwidth = width;

					if(0 == (index % 2))
					{
						writebmplzw1(picname, index, width + bufwid, height, false);
					}
					else
					{
						writebmplzw1(picname, index, (width + bufwid) / 4, height, true);
					}
				}
			}
			free(picarray);
		}
	}
	getch();
}

//-----------------------------------------------------------------------------
// Name: ~PICTURES()
// Desc: Frees the array of pixels
//-----------------------------------------------------------------------------

PICTURES::~PICTURES()
{
	if(initialized == false)
	{
		return;
	}
	for(int a = 0; a < width; a++)
	{
		free(pixels[a]);
	}
	free(pixels);
}

//-----------------------------------------------------------------------------
// Name: ~PICTURES()
// Desc: Frees the array of pixels
//-----------------------------------------------------------------------------

void PICTURES::init()
{
	int tempw = width;
	if(tempw % 2 != 0)
	{
		tempw++;
	}

	pixels = (int**)malloc(sizeof(int) * tempw);
	for(int a = 0; a < tempw; a++)
	{
		pixels[a] = (int*)malloc(sizeof(int) * height);
	}
	initialized = true;
}

//-----------------------------------------------------------------------------
// Name: writezeros
// Desc: Write zeros to the file
//-----------------------------------------------------------------------------

void PICTURES::writezeros(int numzero)
{
	for(int a = 0; a < numzero; a++){
		fputc(0x00,fw);
	}
}